/* ------------------triang.h ---------------------*/
#ifndef TRIANG_H
#define TRIANG_H

/* CONSTANTES */

#define NPMAX    2000          /* Max # of points    */
#define NDMAX    4000          /* Max # of diagonals */

/* TYPES */

typedef struct PointStructure {
  int     x;                  /* abcisse  */        
  int     y;                  /* ordonnee */
}Point;

struct CellStructure{
  struct CellStructure  *prec; /* precedent  */ 
  struct CellStructure  *suiv; /* suivant */ 
  Point *val;                  /* point de E2 */
  int ore;                     /* 1 si oreille, 0 sinon */
};

typedef struct CellStructure cell;
typedef cell *dliste_points;



/* ------------------VARIABLES GLOBALES ---------------------*/

extern int NP;                     /* nombre de points                */
extern Point *POLYG[NPMAX];        /* tableau des sommets du polygone */
extern Point *DIAG[NDMAX];         /* tableau des diagonales          */
extern dliste_points PTS;          /* d-liste des Point               */
extern int xmin,xmax,ymin,ymax;    /* coordonnees extremes            */

/* ---------------------FONCTIONS -----------------------*/
/* ALLOCATION MEMOIRE */
extern Point *Palloc(void);         /* renvoie un pointeur sur Point */
extern cell  *caloc(void);          /* renvoie un pointeur sur cell  */
extern int   *ialloc(void);         /* renvoie un pointeur sur int   */

/* d-LISTES */
extern void ajouter(dliste_points *L, cell *p, Point *P);    /* ajoute P dans L,*/
                                                             /* dans la place apres p */
extern void supprimer(dliste_points *L, cell *p);            /* supprime la place p dans L */

/* ENTREES-SORTIES */
extern void lire_points(void);      /* remplit la d-liste PTS , affecte la var NP */
extern void ecrire_points(void);    /* ecrit les points, un par ligne */
extern void ecrire_diags(void);     /* ecrit les diagonales, une par ligne */
extern void ecrire_ps(void);        /* imprime, sur la sortie standard, le fichier postscript */

/* GEOMETRIE */
extern int det(Point *p1, Point *p2);       /* determinant */
extern int direct(Point *p1, Point *p2, Point *p3); 
           /* renvoie 1 si (*p2-*p1,*p3-*p2) est direct ou co-lineaire  */ 
           /* renvoie 0 si (*p2-*p1,*p3-*p2) est indirect */
extern void changer_coord(Point *OR);
/* transforme les coordonnes en prenant l'origine OR */
extern int  oreille(cell *p); 
/* renvoie 1 ssi p est la pointe d'une oreille, 0 sinon */
extern void trianguler(void);
/* entree: d-liste PTS qui s'enroule dans le sens direct */
/* sortie: tableau DIAG contient NP-3 diagonales qui forment une triangulation */
#endif
